function [corrMat,t,lag,corrMat0,corrMat1,corrMat2,norm2x1,norm2x2] = timeCoherence(x,y,fsamp,maxLagTime,maxAdvTime)
%
% function [corrMat,t,lag,corrMat0,corrMat1,corrMat2,,norm2x1,norm2x2] = timeCoherence(x,y,fsamp,maxLagTime,maxAdvTime)
%  
% Compute correlation  versus lagged signal over cycle.
%  
% INPUTS:
% x, y: input and output in format of structure arrays x{}, y{}, with x{k} the times series of cycle k.
% fsamp: sampling rate
% maxLagTime:  maximal lag to consider  
% maxAdvTime:  maximal negative lag to consider  
%
% OUTPUTS:
% corrMat[i][j]: normalized correlation at time i and lag j, both sampled at
% sample rate fsample. Lag ins seconds ranges from -maxAdvTime to
% maxLagTime.
% t:  array of corresponding times in cycle
% lag: array of corresponding lag times
% corrMat0:  unnormalized correlation  
% corrMat1:  normalized by input x1 only
% corrMat2:  normalized by output x2 only
% norm2x1 and norm2x2 are normalization factors for input and output,
% useful if you want to assemble correlation over multiple data sets.
%
% Author: Kees van den Doel kvdoel@cs.ubc.ca
  
  
% make steps equal and save scalefactors
ncycles = length(x);
nmean = 0;
for s=1:ncycles
  nmean = nmean + length(x{s});
end
nmean = round(nmean/ncycles); % mean number of samples in cycle
tmean = nmean/fsamp; % mean in s

% compute rescaled data, and scale factors (new length/old length)
x1= zeros(nmean,ncycles);
x2= zeros(nmean,ncycles);
scaleFactors= zeros(ncycles);

resampleN = 3; % reduced from default value 10 for faster execution
for s=1:ncycles
  x1(:,s) = resample(x{s},nmean,length(x{s}),resampleN);
  x2(:,s) = resample(y{s},nmean,length(y{s}),resampleN);
  scaleFactors(s) = nmean/length(x{s});
end
  
tmp = size(x1);
noTrials = tmp(2);
trialLength = tmp(1);

maxOffset = fix(fsamp*maxLagTime);
minOffset = fix(fsamp*maxAdvTime);

lag = (-minOffset:maxOffset)/fsamp;
t = (1:trialLength)/fsamp;
crossTerm = zeros(trialLength,maxOffset+minOffset+1); % t X offset
norm2x1 = zeros(trialLength,1);
norm2x2 = zeros(trialLength,maxOffset+minOffset+1);
x1 = (x1' - ones(noTrials,1) * mean(x1'))';
x2 = (x2' - ones(noTrials,1) * mean(x2'))';

for i=1:noTrials
  y1=x1(:,i);
  y2=x2(:,i);  
  for lagn = 1:maxOffset+minOffset+1
    [offset1, offset2, lam] = lagIndexToOffset(lagn,scaleFactors(i),maxOffset,minOffset);
    y2shifted = lam*circshift(y2,-offset1) + (1-lam)*circshift(y2,-offset2);
    crossTerm(:,lagn) = crossTerm(:,lagn) + y1 .* y2shifted;
    norm2x2(:,lagn) =  norm2x2(:,lagn) + y2shifted.^2;
  end
  norm2x1 =  norm2x1 + y1.^2;
end

% compute normalizer
normMat = zeros(trialLength,maxOffset+minOffset+1); % t X offset

for lagn = 1:maxOffset+minOffset+1
  normMat(:,lagn) = 1./(norm2x1.*norm2x2(:,lagn));
  normMat1(:,lagn) = 1./(norm2x1);
  normMat2(:,lagn) = 1./(norm2x2(:,lagn));
end

corrMat0 = crossTerm';
corrMat = crossTerm .* sqrt(normMat);
corrMat = corrMat';
corrMat1 = crossTerm .* sqrt(normMat1);
corrMat1 = corrMat1';
corrMat2 = crossTerm .* sqrt(normMat2);
corrMat2 = corrMat2';

end

function [offset1, offset2, lam] = lagIndexToOffset(lagIndex,scale, ...
                                                    maxOffset,minOffset)
  offsetRaw = lagIndex-minOffset-1;
  offset1 = floor(offsetRaw*scale);
  offset2 = offset1+1;
  lam = -offsetRaw*scale+offset2;
  end
  
  
